"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const util = require("util");
const deps_1 = require("./deps");
class StreamOutput {
    constructor(stream, options = {}) {
        this.stream = stream;
        this.options = options;
        this.output = '';
    }
    static logToFile(msg, logfile) {
        try {
            deps_1.deps.fs.mkdirpSync(path.dirname(logfile));
            deps_1.deps.fs.appendFileSync(logfile, deps_1.deps.stripAnsi(msg));
        }
        catch (err) {
            console.error(err);
        }
    }
    write(msg, options = {}) {
        msg = msg || '';
        const log = options.log !== false;
        if (log)
            this.writeLogFile(msg, StreamOutput.startOfLine);
        // conditionally show timestamp if configured to display
        if (StreamOutput.startOfLine && this.displayTimestamps) {
            msg = this.timestamp(msg);
        }
        if (this.stream) {
            this.stream.write(msg);
        }
        else {
            this.output += deps_1.deps.stripAnsi(msg);
        }
        StreamOutput.startOfLine = msg.endsWith('\n');
    }
    log(data, ...args) {
        let msg = data ? util.format(data, ...args) : '';
        msg += '\n';
        this.write(msg);
    }
    writeLogFile(msg, withTimestamp) {
        if (!this.logfile) {
            return;
        }
        msg = withTimestamp ? this.timestamp(msg) : msg;
        StreamOutput.logToFile(msg, this.logfile);
    }
    timestamp(msg) {
        return `[${deps_1.deps.moment().format()}] ${msg}`;
    }
}
StreamOutput.startOfLine = false;
exports.StreamOutput = StreamOutput;
